package eu.dnetlib.repo.manager.service.controllers;

import eu.dnetlib.api.functionality.ValidatorService;
import eu.dnetlib.api.functionality.ValidatorServiceException;
import eu.dnetlib.domain.functionality.validator.StoredJob;
import eu.dnetlib.repo.manager.shared.JobsOfUser;
import eu.dnetlib.repo.manager.shared.Constants;
import gr.uoa.di.driver.util.ServiceLocator;
import org.apache.log4j.Logger;
import org.json.JSONException;
import org.springframework.stereotype.Component;
import javax.annotation.Resource;

@Component
public class MonitorApiImpl implements MonitorApi {

    @Resource(name = "validatorServiceLocator")
    private ServiceLocator<ValidatorService> validatorServiceLocator;

    private ValidatorService getValidationService() {
        return this.validatorServiceLocator.getService();
    }

    public ServiceLocator<ValidatorService> getValidatorServiceLocator() {
        return validatorServiceLocator;
    }

    public void setValidatorServiceLocator(ServiceLocator<ValidatorService> validatorServiceLocator) {
        this.validatorServiceLocator = validatorServiceLocator;
    }


    private static final Logger LOGGER = Logger
            .getLogger(MonitorApiImpl.class);

    @Override
    public JobsOfUser getJobsOfUser(String user,
                                         String jobType,
                                         String offset,
                                         String limit,
                                         String dateFrom,
                                         String dateTo,
                                         String validationStatus,
                                         String includeJobsTotal) throws JSONException, ValidatorServiceException {

        LOGGER.debug("Getting jobs of user : " + user);
        JobsOfUser retJobs = new JobsOfUser();
        retJobs.setJobs(getValidationService().getStoredJobsNew(user, jobType, Integer.parseInt(offset),
                Integer.parseInt(limit), dateFrom, dateTo, validationStatus));
        if (Boolean.parseBoolean(includeJobsTotal)) {
            retJobs.setTotalJobs(this.getJobsTotalNumberOfUser(user, jobType, null));
            retJobs.setTotalJobsSuccessful(this.getJobsTotalNumberOfUser(user, jobType, Constants.VALIDATION_JOB_STATUS_SUCCESSFUL));
            retJobs.setTotalJobsFailed(this.getJobsTotalNumberOfUser(user, jobType, Constants.VALIDATION_JOB_STATUS_FAILED));
            retJobs.setTotalJobsOngoing(this.getJobsTotalNumberOfUser(user, jobType,Constants.VALIDATION_JOB_STATUS_ONGOING));
        }
        return retJobs;

    }

    private int getJobsTotalNumberOfUser(String user, String jobType, String validationStatus) throws ValidatorServiceException {
        return  getValidationService().getStoredJobsTotalNumberNew(user, jobType, validationStatus);
    }

    @Override
    public int getJobsOfUserPerValidationStatus(String user,
                                                String jobType,
                                                String validationStatus) throws JSONException {
        LOGGER.debug("Getting job with validation status : " + validationStatus);
        try {
            return getValidationService().getStoredJobsTotalNumberNew(user, jobType, validationStatus);
        } catch (ValidatorServiceException e) {
            e.printStackTrace();
        }
        return 0;
    }

    @Override
    public StoredJob getJobSummary(String jobId,
                                   String groupBy) throws JSONException {
        LOGGER.debug("Getting job summary with id : " + jobId);
        try {
            return getValidationService().getStoredJob(Integer.parseInt(jobId), groupBy);
        } catch (ValidatorServiceException e) {
            e.printStackTrace();
        }
        return null;
    }

}
